﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;


using ThinkGeo.MapSuite.Core;

namespace AdjustedScalingImageStyle
{
    class AdjustedScalingImageStyle : Style
    {
        
      private GeoImage image = null;
            private GeographyUnit mapUnit;
            private double maximumScale;
            private double minimumScale;
            private double maximumSize;
            private double minimumSize;
            private float xOffsetInPixel;
            private float yOffsetInPixel;
            private float rotateAngle;
            private int pointingPixelX;
            private int pointingPixelY;

            // A default constructor that just calls the main constructor
            public AdjustedScalingImageStyle()
                : this(GeographyUnit.DecimalDegree,null,double.MaxValue, 0,20,10 )
            {
            }

            // A constructor with fewer parameters that calls the main constructor
            public AdjustedScalingImageStyle(GeographyUnit mapUnit, GeoImage image, double MaximumScale, double MinimumScale, double MaximumSize, double MinimumSize)
               
            {
                this.image = image;
                this.maximumScale = MaximumScale;
                this.minimumScale = MinimumScale;
                this.maximumSize = MaximumSize;
                this.minimumSize = MinimumSize;
                this.mapUnit = mapUnit;
            }


            // This is the map unit to determine the scale.
            public GeographyUnit MapUnit
            {
                get { return mapUnit; }
                set { mapUnit = value; }
            }

            // This is the image that will be scaled.
            public GeoImage Image
            {
                get { return image; }
                set { image = value; }
            }

            // The maximum scale will be the largest scale used to calculate the size of the image.
            // If the scale gets larger than the maximum, then we compute the scaling based on
            // this number instead.  This means that after this scale, the image will not get
            // any smaller no matter how much more you zoom out.
            public double MaximumScale
            {
                get { return maximumScale; }
                set { maximumScale = value; }
            }

            // The minimum scale will be the smallest scale used to calculate the size of the image.
            // If the scale gets smaller than the minimum, then we compute the scaling based on
            // this number instead.  This means that after this scale, the image will not get
            // any larger no matter how much more you zoom in.
            public double MinimumScale
            {
                get { return minimumScale; }
                set { minimumScale = value; }
            }

            // The MaximumSize is the size of the image at MinimumScale and lower.
            public double MaximumSize
            {
                get { return maximumSize; }
                set { maximumSize = value; }
            }

            // The MinimumSize is the size of the image at MaximumScale and higher.
            public double MinimumSize
            {
                get { return minimumSize; }
                set { minimumSize = value; }
            }

            //The pixel in the row (X) from the left where the image should be centered.
            public int PointingPixelX
            {
                get { return pointingPixelX; }
                set { pointingPixelX = value; }
            }

            //The pixel in the column (Y) from the right where the image should be centered.
            public int PointingPixelY
            {
                get { return pointingPixelY; }
                set { pointingPixelY = value; }
            }

            // This is the X offset in pixels for the image.
            public float XOffsetInPixel
            {
                get { return xOffsetInPixel; }
                set { xOffsetInPixel = value; }
            }

            // This is the Y offset in pixels for the image.
            public float YOffsetInPixel
            {
                get { return yOffsetInPixel; }
                set { yOffsetInPixel = value; }
            }

            // This is the angle you would like to rotate the image.
            public float RotationAngle
            {
                get { return rotateAngle; }
                set { rotateAngle = value; }
            }

            // This is the method we needed to override.
            protected override void DrawCore(IEnumerable<Feature> features, GeoCanvas canvas, System.Collections.ObjectModel.Collection<SimpleCandidate> labelsInThisLayer, System.Collections.ObjectModel.Collection<SimpleCandidate> labelsInAllLayers)
            {
                // First let's make sure the image isn't null.
                if (image != null)
                {
                    // Loop through all of the features being passed in to draw.
                    foreach (Feature feature in features)
                    {
                        // Let's make sure the features being passed in are points or multi points.
                        WellKnownType shapeWellKnownType = feature.GetWellKnownType();
                        if (shapeWellKnownType == WellKnownType.Point || shapeWellKnownType == WellKnownType.Multipoint)
                        {
                            double currentScale = ExtentHelper.GetScale(canvas.CurrentWorldExtent, canvas.Width, mapUnit);

                            // Enforce the minimum and maximum scale properties
                            if (currentScale > maximumScale) { currentScale = maximumScale; }
                            if (currentScale < minimumScale) { currentScale = minimumScale; }

                            // Calculate the size of the image based on the scale.
                            //If the scale is between minimum scale and maximum scale, the size of the image will be calculated according
                            //to the maximum scale/minimum scale ratio. 
                            //For example:
                            //Minimum Scale:100,000
                            //Maximum Scale:200,000
                            //Minimum Size:10
                            //Maximum Size:50
                            //If the scale is 150,000 (mid value between 100,000 and 200,000), the size of the image will be 
                            //30 (mid value between 10 and 50).
                            
                            float imageSize = (float) (maximumSize - ((currentScale - minimumScale) * (maximumSize - minimumSize)) /
                                                (maximumScale - minimumScale)); //Scaled width of the image.

                            float imageSizeHeight = (imageSize * Image.GetHeight()) / Image.GetWidth(); //Scaled height of the image.

                            //Pointing pixel location according to the scaled image.
                            float newPointingPixelX = (imageSize * pointingPixelX) / Image.GetWidth();
                            float newPointingPixelY = (imageSizeHeight * pointingPixelY) / Image.GetHeight();

                            // Calls the canvas method to draw the image scaled.
                            PointShape point = new PointShape(feature.GetWellKnownBinary());
                            
                            //Gets the location of the image in screen coordinates according to the scaled image and its adjusted pointing location.
                            ScreenPointF screenPointF = ExtentHelper.ToScreenCoordinate(canvas.CurrentWorldExtent, point, canvas.Width, canvas.Height);
                            float newScreenX = screenPointF.X + ((imageSize / 2) - newPointingPixelX);
                            float newScreenY = screenPointF.Y + (( imageSizeHeight /2) - newPointingPixelY) ;

                            canvas.DrawScreenImage(image, newScreenX, newScreenY, imageSize, imageSizeHeight, DrawingLevel.LevelOne, xOffsetInPixel, yOffsetInPixel, rotateAngle);
                        }
                    }
                }
            }
      }
}
